#!/usr/bin/env python3
###
### Script to check water conservation in the IPSL coupled model
###
##  Warning, to install, configure, run, use any of included software or
##  to read the associated documentation you'll need at least one (1)
##  brain in a reasonably working order. Lack of this implement will
##  void any warranties (either express or implied).  Authors assumes
##  no responsability for errors, omissions, data loss, or any other
##  consequences caused directly or indirectly by the usage of his
##  software by incorrectly or partially configured personal
##
##
## SVN information
#  $Author$
#  $Date$
#  $Revision$
#  $Id$
#  $HeadURL$


## Define Experiment
if False : 
    JobName="TEST-CM72-SIMPLE-ROUTING.12" ; TagName="IPSLCM7" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p86caub" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1970 ; YearEnd = 1979 ; PackFrequency = 10
    ATM = 'ICO40' ; Routing = 'SIMPLE'

if False :
    JobName="TEST-CM72-SIMPLE-ROUTING.10" ; TagName="IPSLCM7" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p86caub" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1860 ; YearEnd = 1869 ; PackFrequency = 10
    ATM = 'ICO40' ; Routing = 'SIMPLE'

if False : 
    JobName="VALID-CM622-LR.01" ; TagName="IPSLCM6" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p86caub" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1890 ; YearEnd = 1899 ; PackFrequency = 10
    ATM = 'LMD144142' ; Routing = 'ORCHIDEE'
    
if True :
    JobName="CM65v420-LR-SKL-pi-05" ; TagName="IPSLCM6" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p48ethe" ; Project="gencmip6" 
    Freq = 'MO' ;  YearBegin = 1890 ; YearEnd = 1899 ; PackFrequency = 10
    ORCA = 'eORCA1.2'  ; ATM = 'LMD144142' ; Routing = 'ORCHIDEE' ; NEMO=4.2 ; OCE_relax = False ; OCE_icb = False ; Coupled = True
    
Coupled = True

import numpy as np
##-- Some physical constants
#-- Earth Radius
Ra = 40.e6 / (2. * np.pi)
#-- Gravity
g = 9.81
#-- Ice density (kg/m3) in LIM3
ICE_rho_ice = 917.0
#-- Snow density (kg/m3) in LIM3
ICE_rho_sno = 330.0
#-- Ocean water density (kg/m3) in NEMO
OCE_rho_liq = 1026.

###
ICO  = False
if 'ICO' in ATM : ICO  = True
LMDZ = False
if 'LMD' in ATM : LMDZ = True

###
## Import system modules
import sys, os, shutil, subprocess

# Where do we run ?
TGCC = False ; IDRIS = False
SysName, NodeName, Release, Version, Machine = os.uname()
if 'irene'   in NodeName : TGCC  = True
if 'jeanzay' in NodeName : IDRIS = True

## Set site specific libIGCM directories, and other specific stuff
if TGCC :
    CPU = subprocess.getoutput ( 'lscpu | grep "Model name"' )
    if "Intel(R) Xeon(R) Platinum" in CPU : Machine = 'irene'
    if "AMD"                       in CPU : Machine = 'irene-amd'
        
    ARCHIVE     = subprocess.getoutput ( f'ccc_home --cccstore   -d {Project} -u {User}' )
    STORAGE     = subprocess.getoutput ( f'ccc_home --cccwork    -d {Project} -u {User}' )
    SCRATCHDIR  = subprocess.getoutput ( f'ccc_home --cccscratch -d {Project} -u {User}' )
    R_IN        = os.path.join ( subprocess.getoutput ( f'ccc_home --cccwork -d igcmg -u igcmg' ), 'IGCM')
    rebuild     = os.path.join ( subprocess.getoutput ( f'ccc_home --ccchome -d igcmg -u igcmg' ), 'Tools', Machine, 'rebuild_nemo', 'bin', 'rebuild_nemo' )

    ## Specific to run at TGCC.
    # Needed before importing a NetCDF library (netCDF4, xarray, cmds, etc ...)
    import mpi4py
    mpi4py.rc.initialize = False
        
    ## Access to the nemo and lmdz module
    sys.path.append ( os.path.join (subprocess.getoutput ( f'ccc_home -u p86mart' ), 'Python', 'Library'))

    ## Creates output directory
    TmpDir = os.path.join ( subprocess.getoutput ( 'ccc_home --cccscratch' ), f'WATER_{JobName}_{YearBegin}_{YearEnd}' )

if IDRIS :
    raise Exception("Pour IDRIS : repertoires et chemins a definir") 

## Import specific module
import nemo, lmdz
## Now import needed scientific modules
import xarray as xr
    
# Output file
FileOut = f'ATM_waterbudget_{JobName}_{YearBegin}_{YearEnd}.out'
f_out = open ( FileOut, mode = 'w' )

# Function to print to stdout *and* output file
def echo (string) :
    print ( string  )
    f_out.write ( string + '\n' )
    f_out.flush ()
    return None
    
## Set libIGCM directories
R_OUT       = os.path.join ( ARCHIVE   , 'IGCM_OUT')
R_BUF       = os.path.join ( SCRATCHDIR, 'IGCM_OUT')

L_EXP = os.path.join (TagName, SpaceName, ExperimentName, JobName)
R_SAVE      = os.path.join ( R_OUT, L_EXP )
R_BUFR      = os.path.join ( R_BUF, L_EXP )
POST_DIR    = os.path.join ( R_BUF, L_EXP, 'Out' )
REBUILD_DIR = os.path.join ( R_BUF, L_EXP, 'REBUILD' )
R_BUF_KSH   = os.path.join ( R_BUFR, 'Out' )
R_FIGR      = os.path.join ( STORAGE, 'IGCM_OUT', L_EXP )

#if os.path.isdir (TmpDir) : shutil.rmtree ( TmpDir )
if not os.path.isdir (TmpDir) : os.mkdir (TmpDir)
TmpDirOCE = os.path.join (TmpDir, 'OCE')
TmpDirICE = os.path.join (TmpDir, 'ICE')
if not os.path.exists (TmpDirOCE) : os.mkdir (TmpDirOCE )
if not os.path.exists (TmpDirICE) : os.mkdir (TmpDirICE )

echo ( f'Working in TMPDIR : {TmpDir}' )

echo ( f'\nDealing with {L_EXP}' )

#-- Model output directories
if Freq == "MO" : FreqDir =  os.path.join ('Output' , 'MO' )
if Freq == "SE" : FreqDir =  os.path.join ('Analyse', 'SE' )
dir_ATM_his = os.path.join ( R_SAVE, "ATM", FreqDir )
dir_SRF_his = os.path.join ( R_SAVE, "SRF", FreqDir )

echo ( f'The analysis relies on files from the following model output directories : ' )
echo ( f'{dir_ATM_his}' )
echo ( f'{dir_SRF_his}' )

#-- Files Names
if Freq == 'MO' : File = f'{JobName}_{YearBegin}0101_{YearEnd}1231_1M'
if Freq == 'SE' : File = f'{JobName}_SE_{YearBegin}0101_{YearEnd}1231_1M'

echo ('\nOpen history files' )
file_ATM_his = os.path.join (  dir_ATM_his, f'{File}_histmth.nc' )
file_SRF_his = os.path.join (  dir_SRF_his, f'{File}_sechiba_history.nc' )

d_ATM_his = xr.open_dataset ( file_ATM_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_SRF_his = xr.open_dataset ( file_SRF_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()

echo ( file_ATM_his )
echo ( file_SRF_his )

## Compute run length
dtime = ( d_ATM_his.time_counter_bounds.max() - d_ATM_his.time_counter_bounds.min() )
echo ('\nRun length : {:8.2f} days'.format ( (dtime/np.timedelta64(1, "D")).values ) )
dtime_sec = (dtime/np.timedelta64(1, "s")).values.item() # Convert in seconds

## Compute length of each period
dtime_per = (d_ATM_his.time_counter_bounds[:,-1] -  d_ATM_his.time_counter_bounds[:,0] )
echo ('\nPeriods lengths (days) : {:} days'.format ( (dtime_per/np.timedelta64(1, "D")).values ) )
dtime_per_sec = (dtime_per/np.timedelta64(1, "s")).values # In seconds
dtime_per_sec = xr.DataArray (dtime_per_sec, dims=["time_counter", ], coords=[d_ATM_his.time_counter,] )

#-- Open restart files
YearRes = YearBegin - 1        # Year of the restart of beginning of simulation
YearPre = YearBegin - PackFrequency  # Year to find the tarfile of the restart of beginning of simulation

echo (f'Restart dates - Start : {YearRes}-12-31  /  End : {YearEnd}-12-31 ')

file_restart_beg = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{YearPre}0101_{YearRes}1231_restart.tar' )
file_restart_end = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{YearBegin}0101_{YearEnd}1231_restart.tar' )

echo ( f'{file_restart_beg}' )
echo ( f'{file_restart_end}' )

file_ATM_beg = f'ATM_{JobName}_{YearRes}1231_restartphy.nc'
file_ATM_end = f'ATM_{JobName}_{YearEnd}1231_restartphy.nc'
if LMDZ :
    file_DYN_beg = f'ATM_{JobName}_{YearRes}1231_restart.nc'
    file_DYN_end = f'ATM_{JobName}_{YearEnd}1231_restart.nc'
if ICO :
    file_DYN_beg = f'ICO_{JobName}_{YearRes}1231_restart.nc'
    file_DYN_end = f'ICO_{JobName}_{YearEnd}1231_restart.nc'
file_SRF_beg = f'SRF_{JobName}_{YearRes}1231_sechiba_rest.nc'
file_SRF_end = f'SRF_{JobName}_{YearEnd}1231_sechiba_rest.nc'
liste_beg = [file_ATM_beg, file_DYN_beg, file_SRF_beg, ]
liste_end = [file_ATM_end, file_DYN_end, file_SRF_end, ]
echo ( f'{file_ATM_beg}')
echo ( f'{file_ATM_end}')
echo ( f'{file_SRF_beg}')
echo ( f'{file_SRF_end}')

if Routing == 'SIMPLE' : 
    file_RUN_beg = f'SRF_{JobName}_{YearRes}1231_routing_restart.nc'
    file_RUN_end = f'SRF_{JobName}_{YearEnd}1231_routing_restart.nc'
    liste_beg.append ( file_RUN_beg )
    liste_end.append ( file_RUN_end )
    echo ( f'{file_RUN_beg}')
    echo ( f'{file_RUN_end}')

echo ('\nExtract restart files from tar : ATM, ICO and SRF')
for resFile in liste_beg :
    if not os.path.exists ( os.path.join (TmpDir, resFile) ) :
        command =  f'cd {TmpDir} ; tar xf {file_restart_beg} {resFile}'
        echo ( command )
        os.system ( command )
        
for resFile in liste_end :
    if not os.path.exists ( os.path.join (TmpDir, resFile) ) :
        command =  f'cd {TmpDir} ; tar xf {file_restart_end} {resFile}'
        echo ( command )
        os.system ( command )

echo ('\nOpening ATM SRF and ICO restart files')
d_ATM_beg = xr.open_dataset ( os.path.join (TmpDir, file_ATM_beg), decode_times=False, decode_cf=True).squeeze()
d_ATM_end = xr.open_dataset ( os.path.join (TmpDir, file_ATM_end), decode_times=False, decode_cf=True).squeeze()
d_SRF_beg = xr.open_dataset ( os.path.join (TmpDir, file_SRF_beg), decode_times=False, decode_cf=True).squeeze()
d_SRF_end = xr.open_dataset ( os.path.join (TmpDir, file_SRF_end), decode_times=False, decode_cf=True).squeeze()
d_DYN_beg = xr.open_dataset ( os.path.join (TmpDir, file_DYN_beg), decode_times=False, decode_cf=True).squeeze()
d_DYN_end = xr.open_dataset ( os.path.join (TmpDir, file_DYN_end), decode_times=False, decode_cf=True).squeeze()

for var in d_SRF_beg.variables :
    #d_SRF_beg[var].attrs['_FillValue'] = 1.e20
    #d_SRF_end[var].attrs['_FillValue'] = 1.e20
    d_SRF_beg[var] = d_SRF_beg[var].where (  d_SRF_beg[var]<1.e20, 0.)
    d_SRF_end[var] = d_SRF_end[var].where (  d_SRF_end[var]<1.e20, 0.)

if ICO :
    d_RUN_beg = xr.open_dataset ( os.path.join (TmpDir, file_RUN_beg), decode_times=False, decode_cf=True).squeeze()
    d_RUN_end = xr.open_dataset ( os.path.join (TmpDir, file_RUN_end), decode_times=False, decode_cf=True).squeeze()
    
echo ( file_ATM_beg )
echo ( file_ATM_end )
echo ( file_DYN_beg )
echo ( file_DYN_end )
echo ( file_SRF_beg )
echo ( file_SRF_end )
if Routing == 'SIMPLE' :
    echo ( file_RUN_beg )
    echo ( file_RUN_end )

# ATM grid with cell surfaces
if ICO : 
    ATM_aire = d_ATM_his ['aire'][0]
    ATM_fsea = d_ATM_his ['fract_ter'][Ø] + d_ATM_his ['fract_sic'][Ø]
if LMDZ :
    ATM_aire = lmdz.geo2point ( d_ATM_his ['aire'][0] )
    ATM_fsea = lmdz.geo2point ( d_ATM_his ['fract_ter'][0] + d_ATM_his ['fract_sic'][0] )
    ATM_aire[0]  = np.sum ( d_ATM_his ['aire'][0, 0] )
    ATM_aire[-1] = np.sum ( d_ATM_his ['aire'][0,-1] )

if ICO :
    file_DYN_aire = os.path.join ( R_IN, 'ATM', 'GRID', ATM+'_grid.nc' )
    d_DYN_aire = xr.open_dataset ( file_DYN_aire, decode_times=False).squeeze()
    d_DYN_aire = d_DYN_aire.rename( {'cell':'cell_mesh'})
    DYN_aire   = d_DYN_aire['aire']
if LMDZ :
    DYN_aire = ATM_aire

#if LMDZ :
#    d_ATM_beg = d_ATM_beg.assign ( coords={'lon':d_ATM_beg.lon*180./np.pi} )

ATM_aire_tot = np.sum (ATM_aire).values.item()
ATM_aire_sea_tot = np.sum (ATM_aire*ATM_fsea).values.item()

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- LMDZ changes in stores (for the records)' )

#-- Change in precipitable water from the atmosphere daily and monthly files
#-- Compute sum weighted by gridcell area (kg/m2) then convert to Sv

# ATM vertical grid
ATM_Ahyb = d_ATM_his['Ahyb'].squeeze()
ATM_Bhyb = d_ATM_his['Bhyb'].squeeze()
klevp1 = ATM_Ahyb.shape[0]

# Surface pressure
if ICO : 
    ATM_ps_beg = d_DYN_beg['ps']
    ATM_ps_end = d_DYN_end['ps']
    
if LMDZ : 
    ATM_ps_beg =  lmdz.geo2point ( d_DYN_beg['ps'].isel(rlonv=slice(0,-1)) )
    ATM_ps_end =  lmdz.geo2point ( d_DYN_end['ps'].isel(rlonv=slice(0,-1)) )
    
# 3D Pressure
ATM_p_beg = ATM_Ahyb + ATM_Bhyb * ATM_ps_beg
ATM_p_end = ATM_Ahyb + ATM_Bhyb * ATM_ps_end

# Layer thickness
ATM_sigma_beg = ATM_p_beg[0:-1]*0.
ATM_sigma_end = ATM_p_end[0:-1]*0. 

for k in np.arange (klevp1-1) :
    ATM_sigma_beg[k,:] = (ATM_p_beg[k,:] - ATM_p_beg[k+1,:]) / g
    ATM_sigma_end[k,:] = (ATM_p_end[k,:] - ATM_p_end[k+1,:]) / g

ATM_sigma_beg = ATM_sigma_beg.rename ( {'klevp1':'sigs'} )
ATM_sigma_end = ATM_sigma_end.rename ( {'klevp1':'sigs'} )

##-- Vertical and horizontal integral, and sum of liquid, solid and vapor water phases
if LMDZ :
    try : 
        ATM_wat_beg = lmdz.geo3point ( (d_DYN_beg['H2Ov'] + d_DYN_beg['H2Ol'] + d_DYN_beg['H2Oi']).isel(rlonv=slice(0,-1) ) )
        ATM_wat_end = lmdz.geo3point ( (d_DYN_end['H2Ov'] + d_DYN_end['H2Ol'] + d_DYN_end['H2Oi']).isel(rlonv=slice(0,-1) ) )
    except :
        ATM_wat_beg = lmdz.geo3point ( (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s']).isel(rlonv=slice(0,-1) ) )
        ATM_wat_end = lmdz.geo3point ( (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s']).isel(rlonv=slice(0,-1) ) )
if ICO :
    ATM_wat_beg = ( d_DYN_beg['q'][0:4] ).sum (dim = ['nq',] ).rename ( {'lev':'sigs'} )
    ATM_wat_end = ( d_DYN_end['q'][0:4] ).sum (dim = ['nq',] ).rename ( {'lev':'sigs'} )
    
ATM_mas_wat_beg = np.sum (ATM_sigma_beg * ATM_wat_beg * ATM_aire).values.item()
ATM_mas_wat_end = np.sum (ATM_sigma_end * ATM_wat_end * ATM_aire).values.item()

dATM_mas_wat = ATM_mas_wat_end - ATM_mas_wat_beg

echo ( 'Variation du contenu en eau atmosphere ' )
echo ( 'ATM_mass_beg = {:12.6e} kg - ATM_mass_end = {:12.6e} kg'.format (ATM_mas_wat_beg, ATM_mas_wat_end) )
echo ( 'dMass(atm)   = {:12.3e} kg '.format (dATM_mas_wat) )
echo ( 'dMass(atm)   = {:12.3e} Sv '.format (dATM_mas_wat/dtime_sec*1.E-9) )
echo ( 'dMass(atm)   = {:12.3e}m   '.format (dATM_mas_wat/ATM_aire_sea_tot*1E-3) )

ATM_sno_beg = d_ATM_beg['SNOW01']*d_ATM_beg['FTER']+d_ATM_beg['SNOW02']*d_ATM_beg['FLIC']+d_ATM_beg['SNOW03']*d_ATM_beg['FOCE']+d_ATM_beg['SNOW04']*d_ATM_beg['FSIC']
ATM_sno_end = d_ATM_end['SNOW01']*d_ATM_end['FTER']+d_ATM_end['SNOW02']*d_ATM_end['FLIC']+d_ATM_end['SNOW03']*d_ATM_end['FOCE']+d_ATM_end['SNOW04']*d_ATM_end['FSIC']
if ICO :
   ATM_sno_beg = ATM_sno_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_sno_end = ATM_sno_end.rename ( {'points_physiques':'cell_mesh'} )

ATM_mas_sno_beg = np.sum ( ATM_sno_beg * DYN_aire ).values.item()
ATM_mas_sno_end = np.sum ( ATM_sno_end * DYN_aire ).values.item()

dATM_mas_sno = ATM_mas_sno_end - ATM_mas_sno_beg

echo ( 'Variation du contenu en neige atmosphere ' )
echo ( 'ATM_mas_sno_beg  = {:12.6e} kg  - ATM_mas_sno_end  = {:12.6e} kg'.format (ATM_mas_sno_beg, ATM_mas_sno_end) )
echo ( 'dMass(neige atm) = {:12.3e} kg '.format (dATM_mas_sno) )
echo ( 'dMass(neige atm) = {:12.3e} Sv '.format (dATM_mas_sno/dtime_sec*1E-6/ICE_rho_ice) )
echo ( 'dMass(neige atm) = {:12.3e} m  '.format (dATM_mas_sno/ATM_aire_sea_tot*1E-3) )

echo ( '\nVariation du contenu en eau+neige atmosphere ' )
echo ( 'dMass(eau + neige atm) = {:12.3e} kg '.format (  dATM_mas_wat + dATM_mas_sno) )
echo ( 'dMass(eau + neige atm) = {:12.3e} Sv '.format ( (dATM_mas_wat + dATM_mas_sno)/dtime_sec*1E-9) )
echo ( 'dMass(eau + neige atm) = {:12.3e} m  '.format ( (dATM_mas_wat + dATM_mas_sno)/ATM_aire_sea_tot*1E-3) )

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- SRF changes in routing reservoirs' )

if Routing == 'SIMPLE' :
    RUN_mas_wat_beg = np.sum ( d_RUN_beg ['fast_reservoir'] +  d_RUN_beg ['slow_reservoir'] +  d_RUN_beg ['stream_reservoir']).values.item()
    RUN_mas_wat_end = np.sum ( d_RUN_end ['fast_reservoir'] +  d_RUN_end ['slow_reservoir'] +  d_RUN_end ['stream_reservoir']).values.item()

if Routing == 'ORCHIDEE' : 
    RUN_mas_wat_beg = np.sum ( d_SRF_beg['fastres']  + d_SRF_beg['slowres'] + d_SRF_beg['streamres'] \
                             + d_SRF_beg['floodres'] + d_SRF_beg['lakeres'] + d_SRF_beg['pondres'] ) .values.item()
    RUN_mas_wat_end = np.sum ( d_SRF_end['fastres']  + d_SRF_end['slowres'] + d_SRF_end['streamres'] \
                             + d_SRF_end['floodres'] + d_SRF_end['lakeres'] + d_SRF_end['pondres'] ) .values.item()

dRUN_mas_wat = RUN_mas_wat_end - RUN_mas_wat_beg
    
echo ( '\nWater content in routing ' )
echo ( 'RUN_mas_wat_beg = {:12.6e} kg '.format (RUN_mas_wat_beg) )
echo ( 'RUN_mas_wat_end = {:12.6e} kg '.format (RUN_mas_wat_end) )
echo ( 'dMass(routing)  = {:12.3e} kg '.format(dRUN_mas_wat) )
echo ( 'dMass(routing)  = {:12.3e} Sv '.format(dRUN_mas_wat/dtime_sec*1E-9) )
echo ( 'dMass(routing)  = {:12.3e} m  '.format(dRUN_mas_wat/ATM_aire_sea_tot*1E-3) )
    
SRF_mas_wat_beg = d_SRF_beg['tot_watveg_beg']+d_SRF_beg['tot_watsoil_beg'] + d_SRF_beg['snow_beg']
SRF_mas_wat_end = d_SRF_end['tot_watveg_beg']+d_SRF_end['tot_watsoil_beg'] + d_SRF_end['snow_beg']

if LMDZ : 
    SRF_mas_wat_beg = SRF_mas_wat_beg.rename ( {'y':'points_phyiques'} )
    SRF_mas_wat_end = SRF_mas_wat_end.rename ( {'y':'points_phyiques'} )
if ICO : 
    SRF_mas_wat_beg = SRF_mas_wat_beg.rename ( {'y':'cell_mesh'} )
    SRF_mas_wat_end = SRF_mas_wat_end.rename ( {'y':'cell_mesh'} )  

SRF_mas_wat_beg = np.sum ( SRF_mas_wat_beg * SRF_aire ).values.item()
SRF_mas_wat_end = np.sum ( SRF_mas_wat_end * SRF_aire ).values.item()

dSRF_mas_wat = SRF_mas_wat_end - SRF_mas_wat_beg

echo ( '\nWater content in and surface ' )
echo ( 'SRF_mas_wat_beg  = {:12.6e} kg - SRF_mas_wat_end  = {:12.6e} kg '.format (SRF_mas_wat_beg, SRF_mas_wat_end) )
echo ( 'dMass(water srf) = {:12.3e} kg '.format (dSRF_mas_wat) )
echo ( 'dMass(water srf) = {:12.3e} Sv '.format (dSRF_mas_wat/dtime_sec*1E-9) )
echo ( 'dMass(water srf) = {:12.3e} m  '.format (dSRF_mas_wat/ATM_aire_sea_tot*1E-3) )

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '\nWater content in  ATM + SRF + RUN ' )
echo ( 'mas_wat_beg              = {:12.6e} kg '.format (ATM_mas_wat_beg + ATM_mas_sno_beg + RUN_mas_wat_beg + SRF_mas_wat_beg) )
echo ( 'mas_wat_end              = {:12.6e} kg '.format (ATM_mas_wat_end + ATM_mas_sno_end + RUN_mas_wat_end + SRF_mas_wat_end) )
echo ( 'dMass(water atm+srf+run) = {:12.6e} kg '.format ( dATM_mas_wat   + dATM_mas_sno    + dRUN_mas_wat    + dSRF_mas_wat) )
echo ( 'dMass(water atm+srf+run) = {:12.3e} Sv '.format ((dATM_mas_wat   + dATM_mas_sno    + dRUN_mas_wat    + dSRF_mas_wat)/dtime_sec*1E-9) )
echo ( 'dMass(water atm+srf+run) = {:12.3e} m  '.format ((dATM_mas_wat   + dATM_mas_sno    + dRUN_mas_wat    + dSRF_mas_wat)/ATM_aire_sea_tot*1E-3) )


echo ( "\n-----------------" )
echo ( " Atm -> Oce fluxes" )

ATM_wbilo_sea = np.sum ( lmdz.geo2point (d_ATM_his['wbilo_oce'] + d_ATM_his['wbilo_sic'])*dtime_per_sec*ATM_aire ).values.item()
ATM_runoff    = np.sum ( lmdz.geo2point (d_ATM_his['runofflic'])*ATM_aire*dtime_per_sec ).values.item()
ATM_calving   = np.sum ( lmdz.geo2point (d_ATM_his['fqcalving'])*ATM_aire*dtime_per_sec ).values.item()

echo (' wbilo oce+sic         = {:12.5e} (kg) '.format ( ATM_wbilo_sea ) )
echo (' runoff liq            = {:12.5e} (kg) '.format ( ATM_runoff    ) )
echo (' calving               = {:12.5e} (kg) '.format ( ATM_calving   ) )
echo (' total                 = {:12.5e} (kg) '.format ( ATM_wbilo_sea - ATM_runoff - ATM_calving ) )
