#!/usr/bin/env python3
###
### Script to check water conservation in NEMO
###
##  Warning, to install, configure, run, use any of included software or
##  to read the associated documentation you'll need at least one (1)
##  brain in a reasonably working order. Lack of this implement will
##  void any warranties (either express or implied).  Authors assumes
##  no responsability for errors, omissions, data loss, or any other
##  consequences caused directly or indirectly by the usage of his
##  software by incorrectly or partially configured personal
##
##
## SVN information
#  $Author$
#  $Date$
#  $Revision$
#  $Id$
#  $HeadURL$


## Define Experiment
## -----------------

if False : 
    JobName="TEST-CM72-SIMPLE-ROUTING.12" ; TagName="IPSLCM7" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p86caub" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1970 ; YearEnd = 1979 ; YearInc = 10 ; PackFrequency=10
    #Freq = 'MO' ; YearBegin = 1852 ; YearEnd = 1852 ; PackFrequency = 1
    ORCA = 'eORCA1.2' ; NEMO=3.6 ; OCE_relax = False ; OCE_icb = False ; Coupled = True

if False : 
    JobName="TEST-CM72-SIMPLE-ROUTING.10" ; TagName="IPSLCM7" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p86caub" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1860 ; YearEnd = 1869 ; PackFrequency = 10
    ORCA = 'eORCA1.2' ; NEMO=3.6 ; OCE_relax = False ; OCE_icb = False ; Coupled = True

if False : 
    JobName="VALID-CM622-LR.01" ; TagName="IPSLCM6" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p86caub" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1890 ; YearEnd = 1899 ; PackFrequency = 10
    ORCA = 'eORCA1.2' ; NEMO=3.6 ; OCE_relax = False ; OCE_icb = False ; Coupled = True

if False : 
    JobName="VALID-CM622-SIMPLE-ROUTING" ; TagName="IPSLCM6" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p86caub" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1898 ; YearEnd = 1898 ; PackFrequency = 1
    ORCA = 'eORCA1.2' ; NEMO=3.6 ; OCE_relax = False ; OCE_icb = False ; Coupled = True
    
if False : 
    JobName="VALID-CM622-NORESTART" ; TagName="IPSLCM6" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p86caub" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1890 ; YearEnd = 1899 ; PackFrequency = 10
    ORCA = 'eORCA1.2' ; NEMO=3.6 ; OCE_relax = False ; OCE_icb = False ; Coupled = True

if True :
    JobName="CM65v420-LR-SKL-pi-05" ; TagName="IPSLCM6" ; SpaceName="DEVT" ; ExperimentName="piControl" ; User="p48ethe" ; Project="gencmip6" 
    Freq = 'MO' ; YearBegin = 1890 ; YearEnd = 1899 ; PackFrequency = 10
    ORCA = 'eORCA1.2' ; NEMO=4.2 ; OCE_relax = False ; OCE_icb = False ; Coupled = True
    #/ccc/store/cont003/gencmip6/p48ethe/IGCM_OUT/IPSLCM6/DEVT/piControl/CM65v420-LR-SKL-pi-05

import numpy as np
##-- Some physical constants
#-- Earth Radius
Ra = 40.e6 / (2. * np.pi)
#-- Gravity
g = 9.81
#-- Ice density (kg/m3) in LIM3
ICE_rho_ice = 917.0
#-- Snow density (kg/m3) in LIM3
ICE_rho_sno = 330.0
#-- Ocean water density (kg/m3) in NEMO
OCE_rho_liq = 1026.
#-- Water density in ice pounds in SI3
ICE_rho_pnd = 1000.

##
## Import system modules
import sys, os, shutil, subprocess

# Where do we run ?
TGCC = False ; IDRIS = False
SysName, NodeName, Release, Version, Machine = os.uname ()
if 'irene'   in NodeName : TGCC  = True
if 'jeanzay' in NodeName : IDRIS = True

## Set site specific libIGCM directories, and other specific stuff
if TGCC :
    CPU = subprocess.getoutput ( 'lscpu | grep "Model name"' )
    if "Intel(R) Xeon(R) Platinum" in CPU : Machine = 'irene'
    if "AMD"                       in CPU : Machine = 'irene-amd'
        
    ARCHIVE     = subprocess.getoutput ( f'ccc_home --cccstore   -d {Project} -u {User}' )
    STORAGE     = subprocess.getoutput ( f'ccc_home --cccwork    -d {Project} -u {User}' )
    SCRATCHDIR  = subprocess.getoutput ( f'ccc_home --cccscratch -d {Project} -u {User}' )
    R_IN        = os.path.join ( subprocess.getoutput ( f'ccc_home --cccwork -d igcmg -u igcmg'), 'IGCM')
    rebuild     = os.path.join ( subprocess.getoutput ( f'ccc_home --ccchome -d igcmg -u igcmg'), 'Tools', Machine, 'rebuild_nemo', 'bin', 'rebuild_nemo' )

    ## Specific to run at TGCC.
    # Needed before importing a NetCDF library (netCDF4, xarray, cmds, etc ...)
    import mpi4py
    mpi4py.rc.initialize = False
        
    ## Access to the nemo module
    sys.path.append ( os.path.join (subprocess.getoutput ( f'ccc_home -u p86mart' ), 'Python', 'Library') )

    ## Creates output directory
    TmpDir = os.path.join ( subprocess.getoutput ( 'ccc_home --cccscratch' ), f'WATER_{JobName}_{YearBegin}_{YearEnd}' )

if IDRIS :
    raise Exception ("Pour IDRIS : repertoires et chemins a definir") 
    

## Import specific module
import nemo
## Now import needed scientific modules
import xarray as xr
    
# Output file
FileOut = f'OCE_waterbudget_{JobName}_{YearBegin}_{YearEnd}.out'
f_out = open ( FileOut, mode = 'w' )

# Function to print to stdout *and* output file
def echo (string) :
    print ( string  )
    f_out.write ( string + '\n' )
    f_out.flush ()
    return None
    
## Set libIGCM directories
R_OUT       = os.path.join ( ARCHIVE   , 'IGCM_OUT')
R_BUF       = os.path.join ( SCRATCHDIR, 'IGCM_OUT')

L_EXP = os.path.join (TagName, SpaceName, ExperimentName, JobName)
R_SAVE      = os.path.join ( R_OUT, L_EXP )
R_BUFR      = os.path.join ( R_BUF, L_EXP )
POST_DIR    = os.path.join ( R_BUF, L_EXP, 'Out' )
REBUILD_DIR = os.path.join ( R_BUF, L_EXP, 'REBUILD' )
R_BUF_KSH   = os.path.join ( R_BUFR, 'Out' )
R_FIGR      = os.path.join ( STORAGE, 'IGCM_OUT', L_EXP )
    
#if os.path.isdir (TmpDir) : shutil.rmtree ( TmpDir )
if not os.path.isdir (TmpDir) : os.mkdir (TmpDir)
TmpDirOCE = os.path.join (TmpDir, 'OCE')
TmpDirICE = os.path.join (TmpDir, 'ICE')
if not os.path.exists (TmpDirOCE) : os.mkdir (TmpDirOCE )
if not os.path.exists (TmpDirICE) : os.mkdir (TmpDirICE )

echo ( f'Working in TMPDIR : {TmpDir}' )

echo ( f'\nDealing with {L_EXP}' )

#-- Model output directories
if Freq == "MO" : FreqDir =  os.path.join ('Output' , 'MO' )
if Freq == "SE" : FreqDir =  os.path.join ('Analyse', 'SE' )
dir_OCE_his = os.path.join ( R_SAVE, "OCE", FreqDir )
dir_ICE_his = os.path.join ( R_SAVE, "ICE", FreqDir )

echo ( f'The analysis relies on files from the following model output directories : ' )
echo ( f'{dir_OCE_his}' )
echo ( f'{dir_ICE_his}' )

#-- Files Names
if Freq == 'MO' : File = f'{JobName}_{YearBegin}0101_{YearEnd}1231_1M'
if Freq == 'SE' : File = f'{JobName}_SE_{YearBegin}0101_{YearEnd}1231_1M'

echo ('\nOpen history files' )
file_OCE_his = os.path.join ( dir_OCE_his, f'{File}_grid_T.nc' )
file_OCE_sca = os.path.join ( dir_OCE_his, f'{File}_scalar.nc' )
file_ICE_his = os.path.join ( dir_ICE_his, f'{File}_icemod.nc' )

d_OCE_his = xr.open_dataset ( file_OCE_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_OCE_sca = xr.open_dataset ( file_OCE_sca, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_ICE_his = xr.open_dataset ( file_ICE_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
if NEMO == 3.6 :d_ICE_his = d_ICE_his.rename ( {'y_grid_T':'y', 'x_grid_T':'x'} )

echo ( file_OCE_his )
echo ( file_OCE_sca )
echo ( file_ICE_his )

## Compute run length
dtime = ( d_OCE_his.time_counter_bounds.max() - d_OCE_his.time_counter_bounds.min() )
echo ('\nRun length : {:8.2f} days'.format ( (dtime/np.timedelta64(1, "D")).values ) )
dtime_sec = (dtime/np.timedelta64(1, "s")).values.item() # Convert in seconds

## Compute length of each period
dtime_per = (d_OCE_his.time_counter_bounds[:,-1] -  d_OCE_his.time_counter_bounds[:,0] )
echo ('\nPeriods lengths (days) : ')
echo (' {:}'.format ( (dtime_per/np.timedelta64(1, "D")).values ) )
dtime_per_sec = (dtime_per/np.timedelta64(1, "s")).values # In seconds
dtime_per_sec = xr.DataArray (dtime_per_sec, dims=["time_counter", ], coords=[d_OCE_his.time_counter,] )
dtime_per_sec.attrs['unit'] = 's'

#-- Open restart files
YearRes = YearBegin - 1        # Year of the restart of beginning of simulation
YearPre = YearBegin - PackFrequency  # Year to find the tarfile of the restart of beginning of simulation

echo (f'Restart dates - Start : {YearRes}-12-31  /  End : {YearEnd}-12-31 ')

file_restart_beg = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{YearPre}0101_{YearRes}1231_restart.tar' )
file_restart_end = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{YearBegin}0101_{YearEnd}1231_restart.tar' )

echo ( f'{file_restart_beg}' )
echo ( f'{file_restart_end}' )

file_OCE_beg = f'OCE_{JobName}_{YearRes}1231_restart.nc'
file_OCE_end = f'OCE_{JobName}_{YearEnd}1231_restart.nc'
file_ICE_beg = f'ICE_{JobName}_{YearRes}1231_restart_icemod.nc'
file_ICE_end = f'ICE_{JobName}_{YearEnd}1231_restart_icemod.nc'

echo ( f'{file_OCE_beg}' )
echo ( f'{file_OCE_end}' )
echo ( f'{file_ICE_beg}' )
echo ( f'{file_ICE_end}' )

echo ('\nExtract and rebuild OCE and ICE restarts')
def get_ndomain (zfile) :
    #d_zfile = xr.open_dataset (zfile, decode_times=False).squeeze()
    #ndomain_opa = d_zfile.attrs['DOMAIN_number_total']
    #d_zfile.close ()
    ndomain_opa = subprocess.getoutput ( f'ls {zfile}_*.nc | wc -l' ).format()
    return int (ndomain_opa)

if not os.path.exists ( os.path.join (TmpDir, file_OCE_beg) ) :
    echo ( 'Extracting '+ os.path.join (TmpDir, file_OCE_beg) )
    if not os.path.exists ( os.path.join (TmpDirOCE, f'OCE_{JobName}_{YearRes}1231_restart_0000.nc') ) :
        command =  f'cd {TmpDirOCE} ; tar xf {file_restart_beg}  OCE_{JobName}_{YearRes}1231_restart_*.nc'
        echo ( command )
        os.system ( command )
    print ('1.2')
    echo ('extract ndomain' )
    ndomain_opa = get_ndomain ( os.path.join (TmpDirOCE, f'OCE_{JobName}_{YearRes}1231_restart') )
    command = f'cd {TmpDirOCE} ; {rebuild} OCE_{JobName}_{YearRes}1231_restart {ndomain_opa} ; mv {file_OCE_beg} {TmpDir}'
    echo ( command )
    os.system ( command )
    echo ( f'Rebuild done : {file_OCE_beg}')
    
if not os.path.exists ( os.path.join (TmpDir, file_OCE_end) ) :
    if not os.path.exists ( os.path.join (TmpDirOCE, f'OCE_{JobName}_{YearEnd}1231_restart_0000.nc') ):
        command =  f'cd {TmpDirOCE} ; tar xf {file_restart_end}  OCE_{JobName}_{YearEnd}1231_restart_*.nc'
        echo ( command )
        os.system ( command )
    echo ('extract ndomain' )
    ndomain_opa = get_ndomain ( os.path.join (TmpDirOCE, f'OCE_{JobName}_{YearEnd}1231_restart') )
    command = f'cd {TmpDirOCE} ; {rebuild} OCE_{JobName}_{YearEnd}1231_restart {ndomain_opa} ; mv {file_OCE_end} {TmpDir}'
    echo ( command )
    os.system ( command )
    echo ( f'Rebuild done : {file_OCE_end}')

if not os.path.exists ( os.path.join (TmpDir, file_ICE_beg) ) :
    if not os.path.exists ( os.path.join (TmpDirICE, f'ICE_{JobName}_{YearRes}1231_restart_icemod_0000.nc') ):
        command =  f'cd {TmpDirICE} ; tar xf {file_restart_beg}  ICE_{JobName}_{YearRes}1231_restart_icemod_*.nc'
        echo ( command )
        os.system ( command )
    echo ('extract ndomain' )
    ndomain_opa = get_ndomain ( os.path.join (TmpDirICE, f'ICE_{JobName}_{YearRes}1231_restart_icemod') )
    command = f'cd {TmpDirICE} ; {rebuild} ICE_{JobName}_{YearRes}1231_restart_icemod {ndomain_opa} ; mv {file_ICE_beg} {TmpDir} '
    echo ( command )
    os.system ( command )
    echo ( f'Rebuild done : {file_OCE_beg}')
    
if not os.path.exists ( os.path.join (TmpDir, file_ICE_end) ) :
    if not os.path.exists ( os.path.join (TmpDirICE, f'ICE_{JobName}_{YearEnd}1231_restart_icemod_0000.nc') ):
        command =  f'cd {TmpDirICE} ; tar xf {file_restart_end} ICE_{JobName}_{YearEnd}1231_restart_icemod_*.nc'
        echo ( command )
        os.system ( command )
    echo ('extract ndomain' )
    ndomain_opa = get_ndomain ( os.path.join (TmpDirICE, f'ICE_{JobName}_{YearRes}1231_restart_icemod') )
    command = f'cd {TmpDirICE} ; {rebuild} ICE_{JobName}_{YearEnd}1231_restart_icemod {ndomain_opa} ; mv {file_ICE_end} {TmpDir}'
    echo ( command )
    os.system ( command )
    echo ( f'Rebuild done : {file_ICE_end}')

echo ('Opening OCE and ICE restart files')
if NEMO == 3.6 : 
    d_OCE_beg = xr.open_dataset ( os.path.join (TmpDir, file_OCE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_OCE_end = xr.open_dataset ( os.path.join (TmpDir, file_OCE_end), decode_times=False, decode_cf=True).squeeze()
    d_ICE_beg = xr.open_dataset ( os.path.join (TmpDir, file_ICE_beg), decode_times=False, decode_cf=True).squeeze()
    d_ICE_end = xr.open_dataset ( os.path.join (TmpDir, file_ICE_end), decode_times=False, decode_cf=True).squeeze()
if NEMO == 4.0 or NEMO == 4.2 : 
    d_OCE_beg = xr.open_dataset ( os.path.join (TmpDir, file_OCE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_OCE_end = xr.open_dataset ( os.path.join (TmpDir, file_OCE_end), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_ICE_beg = xr.open_dataset ( os.path.join (TmpDir, file_ICE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_ICE_end = xr.open_dataset ( os.path.join (TmpDir, file_ICE_end), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()

# Get mask and surfaces
sos = d_OCE_his ['sos'][0].squeeze()
OCE_msk = nemo.lbc_mask ( xr.where ( sos>0., 1., 0. ), cd_type = 'T', sval = 0. )

so = sos = d_OCE_his ['sos'][0].squeeze()
OCE_msk3 = nemo.lbc_mask ( xr.where ( so>0., 1., 0. ), cd_type = 'T', sval = 0. )

# lbc_mask removes the duplicate points (periodicity and north fold)
OCE_aire = nemo.lbc_mask ( d_OCE_his ['area'] * OCE_msk, cd_type = 'T', sval = 0.0 )
ICE_aire = OCE_aire

OCE_aire_tot = np.sum (OCE_aire).values.item()
ICE_aire_tot = np.sum (ICE_aire).values.item()


echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- NEMO change in stores (for the records)' )
#-- Note that the total number of days of the run should be diagnosed rather than assumed
#-- Here the result is in Sv
#
#-- Change in ocean volume in freshwater equivalent

OCE_ssh_beg = d_OCE_beg['sshn']
OCE_ssh_end = d_OCE_end['sshn']
OCE_sum_ssh_beg = np.sum ( OCE_ssh_beg * OCE_aire ).values.item()
OCE_sum_ssh_end = np.sum ( OCE_ssh_end * OCE_aire ).values.item()

if NEMO == 3.6 :
    OCE_e3tn_beg = d_OCE_beg['fse3t_n']
    OCE_e3tn_end = d_OCE_end['fse3t_n']
    OCE_sum_e3tn_beg = np.sum ( OCE_e3tn_beg * OCE_aire * OCE_msk3).values.item()
    OCE_sum_e3tn_end = np.sum ( OCE_e3tn_end * OCE_aire * OCE_msk3).values.item()

echo ( 'OCE_sum_ssh_beg = {:12.6e} m^3  - OCE_sum_ssh_end = {:12.6e} m^3'.format (OCE_sum_ssh_beg, OCE_sum_ssh_end) )
dOCE_ssh_vol = ( OCE_sum_ssh_end - OCE_sum_ssh_beg )
dOCE_ssh_mas = dOCE_ssh_vol * OCE_rho_liq

if NEMO == 3.6 :
    echo ( 'OCE_sum_e3tn_beg = {:12.6e} m^3  - OCE_sum_e3tn_end = {:12.6e} m^3'.format (OCE_sum_e3tn_beg, OCE_sum_e3tn_end) )
    dOCE_e3tn_vol = ( OCE_sum_e3tn_end - OCE_sum_e3tn_beg )
    dOCE_e3tn_mas = dOCE_e3tn_vol * OCE_rho_liq

dOCE_vol_wat = dOCE_ssh_vol ; dOCE_mas_wat = dOCE_ssh_mas

echo ( 'dOCE vol    = {:12.3e} m^3'.format ( dOCE_vol_wat) )
echo ( 'dOCE ssh    = {:12.3e} m  '.format ( dOCE_vol_wat/OCE_aire_tot) )
echo ( 'dOCE mass   = {:12.3e} kg '.format ( dOCE_mas_wat) )
echo ( 'dOCE mass   = {:12.3e} Sv '.format ( dOCE_mas_wat/dtime_sec*1E-9) )

if NEMO == 3.6 :
    echo ( 'dOCE e3tn   vol    = {:12.3e} m^3'.format ( dOCE_e3tn_vol) )
    echo ( 'dOCE e3tn   mass   = {:12.3e} kg '.format ( dOCE_e3tn_mas) )


## Glace et neige
if NEMO == 3.6 :
    ICE_ice_beg = d_ICE_beg['v_i_htc1']+d_ICE_beg['v_i_htc2']+d_ICE_beg['v_i_htc3']+d_ICE_beg['v_i_htc4']+d_ICE_beg['v_i_htc5']
    ICE_ice_end = d_ICE_end['v_i_htc1']+d_ICE_end['v_i_htc2']+d_ICE_end['v_i_htc3']+d_ICE_end['v_i_htc4']+d_ICE_end['v_i_htc5']

    ICE_sno_beg = d_ICE_beg['v_s_htc1']+d_ICE_beg['v_s_htc2']+d_ICE_beg['v_s_htc3']+d_ICE_beg['v_s_htc4']+d_ICE_beg['v_s_htc5']
    ICE_sno_end = d_ICE_end['v_s_htc1']+d_ICE_end['v_s_htc2']+d_ICE_end['v_s_htc3']+d_ICE_end['v_s_htc4']+d_ICE_end['v_s_htc5']
    
    ICE_pnd_beg = 0.0
    ICE_pnd_end = 0.0
    
    ICE_fzl_beg = 0.0
    ICE_fzl_end = 0.0

    ICE_mas_wat_beg = ICE_ice_beg*ICE_rho_ice + ICE_sno_beg*ICE_rho_sno
    ICE_mas_wat_end = ICE_ice_end*ICE_rho_ice + ICE_sno_end*ICE_rho_sno
    
if NEMO == 4.0 or NEMO == 4.2 :
    ICE_ice_beg = d_ICE_beg['v_i']
    ICE_ice_end = d_ICE_end['v_i']

    ICE_sno_beg = d_ICE_beg['v_s']
    ICE_sno_end = d_ICE_end['v_s']

    ICE_pnd_beg = d_ICE_beg['v_ip'] # Ice ponds
    ICE_pnd_end = d_ICE_end['v_ip']

    ICE_fzl_beg = d_ICE_beg['v_il'] # Frozen Ice Ponds
    ICE_fzl_end = d_ICE_end['v_il']
    
    ICE_mas_wat_beg =  np.sum ( d_ICE_beg['snwice_mass']*ICE_aire ).values.item()
    ICE_mas_wat_end =  np.sum ( d_ICE_end['snwice_mass']*ICE_aire ).values.item()
    
    
ICE_vol_ice_beg = np.sum ( ICE_ice_beg*ICE_aire ).values.item()
ICE_vol_ice_end = np.sum ( ICE_ice_end*ICE_aire ).values.item()

ICE_vol_sno_beg = np.sum ( ICE_sno_beg*ICE_aire ).values.item()
ICE_vol_sno_end = np.sum ( ICE_sno_end*ICE_aire ).values.item()

ICE_vol_pnd_beg = np.sum ( ICE_pnd_beg*ICE_aire ).values.item()
ICE_vol_pnd_end = np.sum ( ICE_pnd_end*ICE_aire ).values.item()

ICE_vol_fzl_beg = np.sum ( ICE_fzl_beg*ICE_aire ).values.item()
ICE_vol_fzl_end = np.sum ( ICE_fzl_end*ICE_aire ).values.item()

#-- Converting to freswater volume
dICE_vol_ice = ICE_vol_ice_end - ICE_vol_ice_beg
dICE_mas_ice = dICE_vol_ice * ICE_rho_ice

dICE_vol_sno = ICE_vol_sno_end - ICE_vol_sno_beg
dICE_mas_sno = dICE_vol_sno * ICE_rho_sno

dICE_vol_pnd = ICE_vol_pnd_end - ICE_vol_pnd_beg
dICE_mas_pnd = dICE_vol_pnd * ICE_rho_pnd

dICE_vol_fzl= ICE_vol_fzl_end - ICE_vol_fzl_beg
dICE_mas_fzl = dICE_vol_fzl * ICE_rho_pnd

if NEMO == 3.6 :
    dICE_mas_wat = dICE_mas_ice + dICE_mas_sno  
    dSEA_mas_wat = dOCE_mas_wat + dICE_mas_ice + dICE_mas_sno

if NEMO == 4.0 or NEMO == 4.2 :
    dICE_mas_wat = ICE_mas_wat_end - ICE_mas_wat_beg
    dSEA_mas_wat = dOCE_mas_wat + dICE_mas_wat

echo ( 'ICE_vol_ice_beg = {:12.6e} m^3 - ICE_vol_ice_end = {:12.6e} m^3'.format (ICE_vol_ice_beg, ICE_vol_ice_end) )
echo ( 'ICE_vol_sno_beg = {:12.6e} m^3 - ICE_vol_sno_end = {:12.6e} m^3'.format (ICE_vol_sno_beg, ICE_vol_sno_end) )
echo ( 'ICE_vol_pnd_beg = {:12.6e} m^3 - ICE_vol_pnd_end = {:12.6e} m^3'.format (ICE_vol_pnd_beg, ICE_vol_pnd_end) )
echo ( 'ICE_vol_fzl_beg = {:12.6e} m^3 - ICE_vol_fzl_end = {:12.6e} m^3'.format (ICE_vol_fzl_beg, ICE_vol_fzl_end) )

echo ( 'dICE_vol_ice   = {:12.3e} m^3'.format (dICE_vol_ice) )
echo ( 'dICE_vol_sno   = {:12.3e} m^3'.format (dICE_vol_sno) )
echo ( 'dICE_vol_pnd   = {:12.3e} m^3'.format (dICE_vol_pnd) )
echo ( 'dICE_mas_ice   = {:12.3e} m^3'.format (dICE_mas_ice) )
echo ( 'dICE_mas_sno   = {:12.3e} m^3'.format (dICE_mas_sno) )  
echo ( 'dICE_mas_pnd   = {:12.3e} m^3'.format (dICE_mas_pnd) )  
echo ( 'dICE_mas_fzl   = {:12.3e} m^3'.format (dICE_mas_fzl) )  


echo ( '\n------------------------------------------------------------')
echo ( 'Variation du contenu en eau ocean + glace ' )
echo ( 'dMass(ocean)   = {:12.6e} kg '.format(dSEA_mas_wat) )
echo ( 'dVol (ocean)   = {:12.3e} Sv '.format(dSEA_mas_wat/dtime_sec*1E-9) )
echo ( 'dVol (ocean)   = {:12.3e} m  '.format(dSEA_mas_wat*1E-3/OCE_aire_tot) )


###
### And now, working on fluxes !!

# if coupled:
# emp_oce = evap - precip (including blowing snow) - calving
# emp_ice = evap - precip (excluding blowing snow)
# emp_ice = wfx_spr(<0) + wfx_sub(>0) + wfx_snw_sub(v4.0.6) - wfx_err_sub(<0)
# runoffs = rivers + icebergs
# empmr = emp_oce - wfx_ice - wfx_snw - wfx_pnd(v4.0.6) - wfx_err_sub - runoffs
# doce+ice = - evap + precip + calving (emp_oce)
#            + rivers + icebergs       (friver+iceberg ou runoffs)
#            + iceshelf                (iceshelf)
#            - emp_ice                 (emp_ice)
# dice = - emp_ice - wfx_snw - wfx_ice - wfx_pnd(v4.0.6) - wfx_err_sub

#OCE_emp = evap - precip (including blowing snow) - calving
#ICE_emp = wfx_spr(<0) + wfx_sub(>0) + wfx_snw_sub(v4.0.6) - wfx_err_sub(<0)

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- Checks in NEMO - from budget_modipsl.sh (Clement Rousset)' )

def OCE_flux_int (flux, aire=OCE_aire, dt=dtime_per_sec) :
    '''Integrate flux on ocean grid'''
    OCE_flux_int  = np.sum ( flux * aire * dt).values.item()
    #OCE_flux_int  = np.sum ( flux.to_masked_array()[:,:] * aire.to_masked_array()[np.newaxis,:,:] * dt.to_masked_array()[:,np.newaxis,np.newaxis])
    return OCE_flux_int

# Read variable and computes integral over space and time
OCE_empmr      = d_OCE_his['wfo']         ; OCE_mas_empmr    = OCE_flux_int ( OCE_empmr    )
OCE_wfob       = d_OCE_his['wfob']        ; OCE_mas_wfob     = OCE_flux_int ( OCE_wfob     )
OCE_emp_oce    = d_OCE_his['emp_oce']     ; OCE_mas_emp_oce  = OCE_flux_int ( OCE_emp_oce  )
OCE_emp_ice    = d_OCE_his['emp_ice']     ; OCE_mas_emp_ice  = OCE_flux_int ( OCE_emp_ice  )
OCE_iceshelf   = d_OCE_his['iceshelf']    ; OCE_mas_iceshelf = OCE_flux_int ( OCE_iceshelf )
OCE_calving    = d_OCE_his['calving']     ; OCE_mas_calving  = OCE_flux_int ( OCE_calving  )
OCE_iceberg    = d_OCE_his['iceberg']     ; OCE_mas_iceberg  = OCE_flux_int ( OCE_iceberg  )
OCE_friver     = d_OCE_his['friver']      ; OCE_mas_friver   = OCE_flux_int ( OCE_friver   )
OCE_runoffs    = d_OCE_his['runoffs']     ; OCE_mas_runoffs  = OCE_flux_int ( OCE_runoffs  )
if NEMO == 4.0 or NEMO == 4.2 :
    OCE_wfxice     = d_OCE_his['vfxice'] ; OCE_mas_wfxice   = OCE_flux_int ( OCE_wfxice )
    OCE_wfxsnw     = d_OCE_his['vfxsnw'] ; OCE_mas_wfxsnw   = OCE_flux_int ( OCE_wfxsnw )
    OCE_wfxsub     = d_OCE_his['vfxsub'] ; OCE_mas_wfxsub   = OCE_flux_int ( OCE_wfxsub )
if NEMO == 3.6 :
    OCE_wfxice     = d_OCE_his['vfxice']/86400.*ICE_rho_ice ; OCE_mas_wfxice   = OCE_flux_int ( OCE_wfxice )
    OCE_wfxsnw     = d_OCE_his['vfxsnw']/86400.*ICE_rho_sno ; OCE_mas_wfxsnw   = OCE_flux_int ( OCE_wfxsnw )
    OCE_wfxsub     = d_OCE_his['vfxsub']/86400.*ICE_rho_sno ; OCE_mas_wfxsub   = OCE_flux_int ( OCE_wfxsub )
# Additional checks
OCE_evap_oce   = d_OCE_his['evap_ao_cea'] ; OCE_mas_evap_oce   = OCE_flux_int ( OCE_evap_oce )
ICE_evap_ice   = d_OCE_his['subl_ai_cea'] ; ICE_mas_evap_ice   = OCE_flux_int ( ICE_evap_ice )
OCE_snow_oce   = d_OCE_his['snow_ao_cea'] ; OCE_mas_snow_oce   = OCE_flux_int ( OCE_snow_oce )
OCE_snow_ice   = d_OCE_his['snow_ai_cea'] ; OCE_mas_snow_ice   = OCE_flux_int ( OCE_snow_ice )
OCE_rain       = d_OCE_his['rain']        ; OCE_mas_rain       = OCE_flux_int ( OCE_rain     )
ICE_wfxsub_err = d_ICE_his['vfxsub_err']  ; ICE_mas_wfxsub_err = OCE_flux_int ( ICE_wfxsub_err )
if NEMO == 4.0 or NEMO == 4.2 :
    ICE_wfxpnd     = d_ICE_his['vfxpnd']     ; ICE_mas_wfxpnd     = OCE_flux_int ( ICE_wfxpnd     )
    ICE_wfxsnw_sub = d_ICE_his['vfxsnw_sub'] ; ICE_mas_wfxsnw_sub = OCE_flux_int ( ICE_wfxsnw_sub )
    ICE_wfxsnw_pre = d_ICE_his['vfxsnw_pre'] ; ICE_mas_wfxsnw_pre = OCE_flux_int ( ICE_wfxsnw_pre )
if NEMO == 3.6 :
    ICE_wfxpnd = 0.0 ; ICE_mas_wfxpnd = 0.0
    ICE_wfxsnw_sub = d_ICE_his['vfxsub']/86400.*ICE_rho_sno  ; ICE_mas_wfxsnw_sub = OCE_flux_int ( ICE_wfxsnw_sub )
    ICE_wfxsnw_pre = d_ICE_his['vfxspr']/86400.*ICE_rho_sno  ; ICE_mas_wfxsnw_pre = OCE_flux_int ( ICE_wfxsnw_pre     )

OCE_wfcorr    = d_OCE_his['wfcorr'] ; OCE_mas_wfcorr  = OCE_flux_int ( OCE_wfcorr )
if OCE_relax  :
    # ssr and fwb are included in emp=>empmr but not in emp_oce (outputed by sea-ice)
    OCE_vflx_fwb = d_OCE_his['vflx_fwb'] ; OCE_mas_vflx_fwb   = OCE_flux_int ( OCE_vflx_fwb )
    OCE_vflx_ssr = d_OCE_his['vflx_ssr'] ; OCE_mas_vflx_ssr   = OCE_flux_int ( OCE_vflx_ssr )
else : 
    OCE_fwb = 0.0 ; OCE_mas_fwb = 0.0
    OCE_ssr = 0.0 ; OCE_mas_ssr = 0.0
if OCE_icb :
    OCE_berg_icb    = d_OCE_his['berg_floating_melt'] ; OCE_mas_berg_icb = OCE_flux_int ( OCE_berg_icb    )
    OCE_calving_icb = d_OCE_his['calving_icb']        ; OCE_mas_calv_icb = OCE_flux_int ( OCE_calving_icb )
else :
    OCE_berg_icb = 0. ; OCE_mas_berg_icb = 0.
    OCE_calv_icb = 0. ; OCE_mas_calv_icb = 0.

OCE_mas_emp = OCE_mas_emp_oce - OCE_mas_wfxice - OCE_mas_wfxsnw - ICE_mas_wfxpnd - ICE_mas_wfxsub_err


echo ('\n-- Fields:' ) 
echo ('  EMPMR      {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_empmr     , OCE_mas_empmr      / dtime_sec*1E-9 ))
echo ('  WFOB       {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_wfob      , OCE_mas_wfob       / dtime_sec*1E-9 ))
echo ('  EMP_OCE    {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_emp_oce   , OCE_mas_emp_oce    / dtime_sec*1E-9 ))
echo ('  EMP_ICE    {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_emp_ice   , OCE_mas_emp_ice    / dtime_sec*1E-9 ))
echo ('  ICEBERG    {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_iceberg   , OCE_mas_iceberg    / dtime_sec*1E-9 ))
echo ('  ICESHELF   {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_iceshelf  , OCE_mas_iceshelf   / dtime_sec*1E-9 ))
echo ('  CALVING    {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_calving   , OCE_mas_calving    / dtime_sec*1E-9 ))
echo ('  FRIVER     {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_friver    , OCE_mas_friver     / dtime_sec*1E-9 ))  
echo ('  RUNOFFS    {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_runoffs   , OCE_mas_runoffs    / dtime_sec*1E-9 ))
echo ('  WFXICE     {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_wfxice    , OCE_mas_wfxice     / dtime_sec*1E-9 ))
echo ('  WFXSNW     {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_wfxsnw    , OCE_mas_wfxsnw     / dtime_sec*1E-9 ))
echo ('  WFXSUB     {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_wfxsub    , OCE_mas_wfxsub     / dtime_sec*1E-9 ))
echo ('  WFXPND     {:12.3e} (kg) {:12.3f} (Sv) '.format ( ICE_mas_wfxpnd    , ICE_mas_wfxpnd     / dtime_sec*1E-9 ))
echo ('  WFXSNW_SUB {:12.3e} (kg) {:12.3f} (Sv) '.format ( ICE_mas_wfxsnw_sub, ICE_mas_wfxsnw_sub / dtime_sec*1E-9 ))
echo ('  WFXSNW_PRE {:12.3e} (kg) {:12.3f} (Sv) '.format ( ICE_mas_wfxsnw_pre, ICE_mas_wfxsnw_pre / dtime_sec*1E-9 ))
echo ('  WFXSUB_ERR {:12.3e} (kg) {:12.3e} (Sv) '.format ( ICE_mas_wfxsub_err, ICE_mas_wfxsub_err / dtime_sec*1E-9 ))
echo ('  EVAP_OCE   {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_evap_oce  , OCE_mas_evap_oce   / dtime_sec*1E-9 ))
echo ('  EVAP_ICE   {:12.3e} (kg) {:12.3f} (Sv) '.format ( ICE_mas_evap_ice  , ICE_mas_evap_ice   / dtime_sec*1E-9 ))
echo ('  SNOW_OCE   {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_snow_oce  , OCE_mas_snow_oce   / dtime_sec*1E-9 ))
echo ('  SNOW_ICE   {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_snow_ice  , OCE_mas_snow_ice   / dtime_sec*1E-9 ))
echo ('  RAIN       {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_rain      , OCE_mas_rain       / dtime_sec*1E-9 ))
echo ('  FWB        {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_fwb       , OCE_mas_fwb        / dtime_sec*1E-9 ))
echo ('  SSR        {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_ssr       , OCE_mas_ssr        / dtime_sec*1E-9 ))
echo ('  WFCORR     {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_wfcorr    , OCE_mas_wfcorr     / dtime_sec*1E-9 ))
echo ('  BERG_ICB   {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_berg_icb  , OCE_mas_berg_icb   / dtime_sec*1E-9 ))
echo ('  CALV_ICB   {:12.3e} (kg) {:12.3f} (Sv) '.format ( OCE_mas_calv_icb  , OCE_mas_calv_icb   / dtime_sec*1E-9 ))

echo     ('\n===>  Comparing ===>' ) 
echo     ('  WFX OCE                     = -empmr + iceshelf                                            = {:12.5e} (kg) '.format ( -OCE_mas_empmr + OCE_mas_iceshelf) )
echo     ('     versus d OCE                                                                            = {:12.5e} (kg) '.format ( dOCE_mas_wat) )
echo     ('  WFX ICE+SNW+PND 1           = emp_ice - wfxice - wfxsnw - wfxpnd - wfxsub_err              = {:12.5e} (kg) '.format ( -OCE_mas_emp_ice - OCE_mas_wfxice - OCE_mas_wfxsnw - ICE_mas_wfxpnd - ICE_mas_wfxsub_err) )
echo     ('  WFX ICE+SNW+PND 2           = -emp_ice + empmr - emp_oce + runoffs                         = {:12.5e} (kg) '.format ( -OCE_mas_emp_ice + OCE_mas_empmr - OCE_mas_emp_oce + OCE_mas_runoffs )) 
echo     ('     versus d ICE+SNW+PND                                                                    = {:12.5e} (kg) '.format ( dICE_mas_wat))  # Manque PND ?
if Coupled : 
    echo ('  WFX OCE+ICE+SNW+PND         = -emp_oce - emp_ice + runoffs + iceshelf                      = {:12.5e} (kg) '.format ( -OCE_mas_emp_oce - OCE_mas_emp_ice + OCE_mas_runoffs + OCE_mas_iceshelf))
else :
    echo ('  WFX OCE+ICE+SNW+PND         = -emp_oce - emp_ice + runoffs + iceberg + calving + iceshelf  = {:12.5e} (kg) '.format ( -OCE_mas_emp_oce - OCE_mas_emp_ice + OCE_mas_runoffs + OCE_mas_iceberg + OCE_mas_calving + OCE_mas_iceshelf))
echo     ('     versus d OCE+ICE+SNW+PND                                                                = {:12.5e} (kg) '.format ( dSEA_mas_wat  )) # Manque PND

echo ('\n===> Leaks ===>')
echo ('   Leak OCE             = dOCE_mas_wat + empmr - iceshelf                                                 = {:12.3e} (kg) '.format ( dOCE_mas_wat + OCE_mas_empmr - OCE_mas_iceshelf) ) 
echo ('                        = (dOCE_mas_wat + empmr - iceshelf)/abs(dOCE_mas_wat)                             = {:12.1e} (-)  '.format ( (dOCE_mas_wat + OCE_mas_empmr - OCE_mas_iceshelf )  / abs (dOCE_mas_wat)  ) )
echo ('   Leak ICE+SNW+PND 1   = dICE_mas_wat + emp_ice + wfxice + wfxsnw + wfxpnd + wfxsub_err                  = {:12.3e} (kg) '.format ( dICE_mas_wat + OCE_mas_emp_ice + OCE_mas_wfxice + OCE_mas_wfxsnw + ICE_mas_wfxpnd + ICE_mas_wfxsub_err ) )
echo ('                        = (dICE_mas_wat + emp_ice + wfxice + wfxsnw + wfxpnd + wfxsub_err )/dICE_mas_wat  = {:12.1e} (-)  '.format ( (dICE_mas_wat + OCE_mas_emp_ice + OCE_mas_wfxice + OCE_mas_wfxsnw + ICE_mas_wfxpnd + ICE_mas_wfxsub_err)/abs(dICE_mas_wat)) )
echo ('   Leak ICE+SNW+PND 2   = dICE_mas_wat + emp_ice - empmr + emp_oce - runoffs                              = {:12.3e} (kg) '.format ( dICE_mas_wat + OCE_mas_emp_ice - OCE_mas_empmr + OCE_mas_emp_oce - OCE_mas_runoffs ))
echo ('                        = (dICE_mas_wat - empmr  + emp_oce - runoffs)/abs(dICE_mas_wat)                   = {:12.1e} (-)  '.format ( (dICE_mas_wat - OCE_mas_empmr  + OCE_mas_emp_oce - OCE_mas_runoffs)/abs(dICE_mas_wat)) )
echo ('   Leak OCE+ICE+SNW+PND =  d(ICE+OCE)_mas_wat + emp_oce + emp_ice - runoffs - iceshelf                    = {:12.3e} (kg) '.format ( dSEA_mas_wat + OCE_mas_emp_oce +OCE_mas_emp_ice - OCE_mas_runoffs - OCE_mas_iceshelf ))
echo ('                        = (dSEA_mas_wat + emp_oce + emp_ice - runoffs - iceshelf)*/abs(dSEA_mas_wat)      = {:12.1e} (-)  '.format ( (dSEA_mas_wat + OCE_mas_emp_oce + OCE_mas_emp_ice - OCE_mas_runoffs - OCE_mas_iceshelf)/abs(dSEA_mas_wat) ) )


# check if emp     = emp_ice + emp_oce - calving
#          emp_ice = wfxsub + wfxsnw_sub + wfxsnw_pre - wfxsub_err

echo     ('\n===> Checks ===>' )
echo     ('   Check EMPMR           = empmr - emp_oce + runoffs + wfxice + wfxsnw + wfxpnd + wfxsub_err = {:12.5e} (kg) '.format ( OCE_mas_empmr - OCE_mas_emp_oce + OCE_mas_runoffs + OCE_mas_wfxice + OCE_mas_wfxsnw + ICE_mas_wfxpnd + ICE_mas_wfxsub_err ))
if Coupled : 
    echo ('   Check EMP_OCE         = emp_oce + fwb + ssr - evap_oce + rain + snow_oce + calving        = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_fwb + OCE_mas_ssr - OCE_mas_evap_oce + OCE_mas_rain + OCE_mas_snow_oce + OCE_mas_calving ))
else :
    echo ('   Check EMP_OCE         = emp_oce + ssr + fwb - evap_oce + rain + snow_oce                  = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_ssr + OCE_mas_fwb - OCE_mas_evap_oce + OCE_mas_rain + OCE_mas_snow_oce ))
echo     ('   Check EMP_ICE         = emp_ice - evap_ice + snow_ice                                     = {:12.5e} (kg) '.format ( OCE_mas_emp_ice - ICE_mas_evap_ice + OCE_mas_snow_ice ))
echo     ('   Check EMP_ICE vs WFX  = emp_ice - wfxsub - wfxsnw_sub - wfxsnw_pre + wfxsub_err           = {:12.5e} (kg) '.format ( OCE_mas_emp_ice - OCE_mas_wfxsub - ICE_mas_wfxsnw_sub - ICE_mas_wfxsnw_pre + ICE_mas_wfxsub_err ))

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- Calculs dans la note PDF de Clement')
echo ( 'Freshwater budget of the ice-ocean system          = emp_oce + emp_ice - runoffs - iceberg - iceshelf                = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_emp_ice - OCE_mas_runoffs - OCE_mas_iceberg - OCE_mas_iceshelf ))
echo ( 'Freshwater budget of the ice-ocean system          = emp_oce + emp_ice - friver  - iceberg - iceshelf                = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_emp_ice - OCE_mas_friver  - OCE_mas_iceberg - OCE_mas_iceshelf ))
echo ( 'Freshwater budget in the ocean = ocean mass change = emp_oce - vfxice - ( vfxsnw + vfxsub_err ) - runoffs - iceshelf = {:12.5e} (kg) '.format ( OCE_mas_emp_oce - OCE_mas_wfxice - ( OCE_mas_wfxsnw + ICE_mas_wfxsub_err ) - OCE_mas_runoffs - OCE_mas_iceshelf ))
echo ( 'Freshwater budget in the ocean = ocean mass change = emp_oce - vfxice - ( vfxsnw + vfxsub_err ) - friver  - iceshelf = {:12.5e} (kg) '.format ( OCE_mas_emp_oce - OCE_mas_wfxice - ( OCE_mas_wfxsnw + ICE_mas_wfxsub_err ) - OCE_mas_friver  - OCE_mas_iceshelf ))
echo ( 'Freshwater budget in the ice = ice mass change     = vfxice + ( vfxsnw + vfxsub + vfxspr )                           = {:12.5e} (kg) '.format ( OCE_mas_wfxice  + OCE_mas_wfxsnw + OCE_mas_wfxsub + ICE_mas_wfxsnw_pre ))
echo ( 'Freshwater flux at the interface [ice/snow]-ocean  = vfxice + ( vfxsnw + vfxsub_err )                                = {:12.5e} (kg) '.format ( OCE_mas_wfxsub  + ICE_mas_wfxsnw_pre ))
echo ( 'Freshwater flux at the interface [ice/snow]-atm    = ( vfxsub + vfxspr )                                             = {:12.5e} (kg) '.format ( OCE_mas_emp_ice + ICE_mas_wfxsub_err ))
echo ( 'Freshwater flux at the interface [ice/snow]-atm    = emp_ice + vfxsub_err                                            = {:12.5e} (kg) '.format ( OCE_mas_emp_ice + ICE_mas_wfxsub_err ))
echo ( 'Freshwater flux at the interface ocean-atm         = emp_oce + calving - vfxsub_err                                  = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_calving - ICE_mas_wfxsub_err ))

echo ( "scsshtot   : global_average_sea_level_change                            = {:12.3e} (m) ".format ( np.sum (d_OCE_sca['scsshtot']  ).values  ) )
echo ( "scsshtot   : global_average_sea_level_change                            = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['scsshtot']  ).values * OCE_aire_tot*OCE_rho_liq  ) )
echo ( "bgvolssh   : drift in global mean ssh volume wrt timestep 1             = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['bgvolssh']  ).values * 1e9 * OCE_rho_liq  ) )
echo ( "bgvole3t   : drift in global mean volume variation (e3t) wrt timestep 1 = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['bgvole3t']  ).values * 1e9 * OCE_rho_liq  ) )
echo ( "bgfrcvol   : global mean volume from forcing                            = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['bgfrcvol']  ).values * 1e9 * OCE_rho_liq  ) )
echo ( "ibgvol_tot : global mean ice volume                                     = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['ibgvol_tot']).values * 1e9 * OCE_rho_liq  ) )
echo ( "sbgvol_tot : global mean snow volume                                    = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['sbgvol_tot']).values * 1e9 * OCE_rho_liq  ) )
echo ( "ibgvolume  : drift in ice/snow volume (equivalent ocean volume)         = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['ibgvolume'] ).values * 1e9 * OCE_rho_liq  ) )

